#  AreaEd - an editor for rom-style areas, especially for
#  Dawn of the Ages  http://dawn.netcologne.de
#
#  Copyright (C) 2001 Peter Herth <p.herth@uni-koeln.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



from Area import *


def isspace(char):
    if char == " " or char =="\t" or char == "\r" or char == "\n":
        return 1
    else:
        return 0 

class Tokenizer:
    def __init__(self,str):
        self.source = str
        self.readpos = 0
        self.max = len(str)

    def read(self):
        if (self.readpos < self.max):
            erg = self.source[self.readpos]
            self.readpos = self.readpos +1
            return erg

    def peek(self, pos):
        return self.source[pos]
    
    def get(self, start, stop):
        #print "start:",start,"stop:",stop
        return self.source[start:stop]

    def readRest(self):
        start = self.readpos
        self.readpos = self.max
        return self.source[start:self.max]

    def readLong(self):
        start = self.readpos
        if (start>= self.max):
            return None
        
        while (start < self.max) and isspace(self.peek(start)):
            start = start + 1

        stop = start
        
        while (stop < self.max) and not self.peek(stop)=="~":
            stop = stop + 1

        self.readpos = stop +1
        return self.get(start,stop)

    def readLine(self):
        start = self.readpos
        if (start>= self.max):
            return None
        stop = start
        while (stop < self.max) and not self.peek(stop)=="\n":
            stop = stop + 1

        self.readpos = stop +1
        return self.get(start,stop)

            

    def readInt(self):
        w = self.readWord()
        i = 0
        try:
            i = int(w)
        except ValueError:
            print "readInt:%s is not a valid integer!" % w
        return i
    
    def readWord(self):
        start = self.readpos
        if (start>= self.max):
            return None
        
        while (start < self.max) and isspace(self.peek(start)):
            start = start + 1

        stop = start
        while (stop < self.max) and not isspace(self.peek(stop)):
            stop = stop + 1

        self.readpos = stop

        return self.get(start, stop)

    def readDelim(self):
        start = self.readpos
        if (start>= self.max):
            return None
        
        while (start < self.max) and isspace(self.peek(start)):
            start = start + 1

        stop = start

        delim = None
        b = self.peek(start)
        if b == "\"" or b=="'" or b=="'":
            delim = b
        if b == "(":
            delim = ")"

        if delim:
            start = start + 1
            stop = start 
            while (stop < self.max) and self.peek(stop)!=delim:
                stop = stop + 1
        else:
            while (stop < self.max) and not isspace(self.peek(stop)):
                stop = stop + 1

        self.readpos = stop + 1

        return self.get(start, stop)
    

def load(fname):
    file = open(fname,"r")

    if (file ==None):
        print "Cannot open file %s!\n" % fname
        return None

    f = file.read(-1)
    file.close()

    print "read %d chars from %s." % (len(f),fname)

    t = Tokenizer(f)
    area = Area()
    area.load(t)
    print "Ok."
    return area


